@@ -122,7 +122,7 @@ class DummyDomain(object):
 
 
 
-class SMTPTestCase(unittest.TestCase):
+class SMTPTests(unittest.TestCase):
 
     messages = [('foo@bar.com', ['foo@baz.com', 'qux@baz.com'], '''\
 Subject: urgent\015
@@ -232,10 +232,10 @@ class LoopbackTestCase(LoopbackMixin):
         return self.loopback(protocol, clientProtocol)
     testMessages.suppress = [util.suppress(message='DomainSMTP', category=DeprecationWarning)]
 
-class LoopbackSMTPTestCase(LoopbackTestCase, unittest.TestCase):
+class LoopbackSMTPTests(LoopbackTestCase, unittest.TestCase):
     clientClass = MySMTPClient
 
-class LoopbackESMTPTestCase(LoopbackTestCase, unittest.TestCase):
+class LoopbackESMTPTests(LoopbackTestCase, unittest.TestCase):
     clientClass = MyESMTPClient
 
 
@@ -266,7 +266,7 @@ class FakeSMTPServer(basic.LineReceiver):
             self.sendLine(self.clientData.pop())
 
 
-class SMTPClientTestCase(unittest.TestCase, LoopbackMixin):
+class SMTPClientTests(unittest.TestCase, LoopbackMixin):
     """
     Tests for L{smtp.SMTPClient}.
     """
@@ -520,11 +520,11 @@ To: foo
         a.setTimeout(None)
 
 
-class AnotherESMTPTestCase(AnotherTestCase, unittest.TestCase):
+class AnotherESMTPTests(AnotherTestCase, unittest.TestCase):
     serverClass = DummyESMTP
     clientClass = MyESMTPClient
 
-class AnotherSMTPTestCase(AnotherTestCase, unittest.TestCase):
+class AnotherSMTPTests(AnotherTestCase, unittest.TestCase):
     serverClass = DummySMTP
     clientClass = MySMTPClient
 
@@ -582,7 +582,7 @@ class DummyRealm:
 
 
 
-class AuthTestCase(unittest.TestCase, LoopbackMixin):
+class AuthTests(unittest.TestCase, LoopbackMixin):
     def test_crammd5Auth(self):
         """
         L{ESMTPClient} can authenticate using the I{CRAM-MD5} SASL mechanism.
@@ -651,12 +651,12 @@ class AuthTestCase(unittest.TestCase, LoopbackMixin):
 
 
 
-class SMTPHelperTestCase(unittest.TestCase):
+class SMTPHelperTests(unittest.TestCase):
     def testMessageID(self):
         d = {}
         for i in range(1000):
             m = smtp.messageid('testcase')
-            self.failIf(m in d)
+            self.assertFalse(m in d)
             d[m] = None
 
     def testQuoteAddr(self):
@@ -729,7 +729,7 @@ class NoticeTLSClient(MyESMTPClient):
 
 
 
-class TLSTestCase(unittest.TestCase, LoopbackMixin):
+class TLSTests(unittest.TestCase, LoopbackMixin):
     if sslSkip is not None:
         skip = sslSkip
 
@@ -747,12 +747,12 @@ class TLSTestCase(unittest.TestCase, LoopbackMixin):
         return self.loopback(server, client).addCallback(check)
 
 if not interfaces.IReactorSSL.providedBy(reactor):
-    for case in (TLSTestCase,):
+    for case in (TLSTests,):
         case.skip = "Reactor doesn't support SSL"
 
 
 
-class EmptyLineTestCase(unittest.TestCase):
+class EmptyLineTests(unittest.TestCase):
     def test_emptyLineSyntaxError(self):
         """
         If L{smtp.SMTP} receives an empty line, it responds with a 500 error
@@ -766,12 +766,12 @@ class EmptyLineTestCase(unittest.TestCase):
 
         out = transport.value().splitlines()
         self.assertEqual(len(out), 2)
-        self.failUnless(out[0].startswith('220'))
+        self.assertTrue(out[0].startswith('220'))
         self.assertEqual(out[1], "500 Error: bad syntax")
 
 
 
-class TimeoutTestCase(unittest.TestCase, LoopbackMixin):
+class TimeoutTests(unittest.TestCase, LoopbackMixin):
     """
     Check that SMTP client factories correctly use the timeout.
     """
@@ -925,7 +925,7 @@ class MultipleDeliveryFactorySMTPServerFactory(protocol.ServerFactory):
 
 
 
-class SMTPSenderFactoryTestCase(unittest.TestCase):
+class SMTPSenderFactoryTests(unittest.TestCase):
     """
     Tests for L{smtp.SMTPSenderFactory}.
     """
@@ -964,7 +964,7 @@ class SMTPSenderFactoryTestCase(unittest.TestCase):
 
 
 
-class SMTPSenderFactoryRetryTestCase(unittest.TestCase):
+class SMTPSenderFactoryRetryTests(unittest.TestCase):
     """
     Tests for the retry behavior of L{smtp.SMTPSenderFactory}.
     """
@@ -1062,7 +1062,7 @@ class NotImplementedDelivery(object):
 
 
 
-class SMTPServerTestCase(unittest.TestCase):
+class SMTPServerTests(unittest.TestCase):
     """
     Test various behaviors of L{twisted.mail.smtp.SMTP} and
     L{twisted.mail.smtp.ESMTP}.
@@ -1261,7 +1261,7 @@ class SMTPServerTestCase(unittest.TestCase):
 
 
 
-class ESMTPAuthenticationTestCase(unittest.TestCase):
+class ESMTPAuthenticationTests(unittest.TestCase):
     def assertServerResponse(self, bytes, response):
         """
         Assert that when the given bytes are delivered to the ESMTP server
@@ -1288,9 +1288,9 @@ class ESMTPAuthenticationTestCase(unittest.TestCase):
         """
         d, credentials, mind, interfaces = loginArgs.pop()
         self.assertEqual(loginArgs, [])
-        self.failUnless(twisted.cred.credentials.IUsernamePassword.providedBy(credentials))
+        self.assertTrue(twisted.cred.credentials.IUsernamePassword.providedBy(credentials))
         self.assertEqual(credentials.username, username)
-        self.failUnless(credentials.checkPassword(password))
+        self.assertTrue(credentials.checkPassword(password))
         self.assertIn(smtp.IMessageDeliveryFactory, interfaces)
         self.assertIn(smtp.IMessageDelivery, interfaces)
         d.callback((smtp.IMessageDeliveryFactory, None, lambda: None))
@@ -1493,7 +1493,7 @@ class ESMTPAuthenticationTestCase(unittest.TestCase):
 
 
 
-class SMTPClientErrorTestCase(unittest.TestCase):
+class SMTPClientErrorTests(unittest.TestCase):
     """
     Tests for L{smtp.SMTPClientError}.
     """
@@ -1562,6 +1562,84 @@ class SenderMixinSentMailTests(unittest.TestCase):
 
 
 
+class ESMTPDowngradeTestCase(unittest.TestCase):
+    """
+    Tests for the ESMTP -> SMTP downgrade functionality in L{smtp.ESMTPClient}.
+    """
+    def setUp(self):
+        self.clientProtocol = smtp.ESMTPClient(
+            b"testpassword", None, b"testuser")
+
+    def test_requireHELOFallbackOperates(self):
+        """
+        If both authentication and transport security are not required, and it
+        is asked for, it will fall back to allowing HELO.
+        """
+        transport = StringTransport()
+        self.clientProtocol.requireAuthentication = False
+        self.clientProtocol.requireTransportSecurity = False
+        self.clientProtocol.heloFallback = True
+        self.clientProtocol.makeConnection(transport)
+
+        self.clientProtocol.dataReceived(b"220 localhost\r\n")
+        transport.clear()
+        self.clientProtocol.dataReceived(b"500 not an esmtp server\r\n")
+        self.assertEqual(b"HELO testuser\r\n", transport.value())
+
+
+    def test_requireAuthFailsHELOFallback(self):
+        """
+        If authentication is required, and HELO fallback is on, HELO fallback
+        must not be honoured, as authentication requires EHLO to succeed.
+        """
+        transport = StringTransport()
+        self.clientProtocol.requireAuthentication = True
+        self.clientProtocol.requireTransportSecurity = False
+        self.clientProtocol.heloFallback = True
+        self.clientProtocol.makeConnection(transport)
+
+        self.clientProtocol.dataReceived(b"220 localhost\r\n")
+        transport.clear()
+        self.clientProtocol.dataReceived(b"500 not an esmtp server\r\n")
+        self.assertEqual("QUIT\r\n", transport.value())
+
+
+    def test_requireTLSFailsHELOFallback(self):
+        """
+        If TLS is required and the connection is insecure, HELO fallback must
+        not be honoured, as STARTTLS requires EHLO to succeed.
+        """
+        transport = StringTransport()
+        self.clientProtocol.requireAuthentication = False
+        self.clientProtocol.requireTransportSecurity = True
+        self.clientProtocol.heloFallback = True
+        self.clientProtocol.makeConnection(transport)
+
+        self.clientProtocol.dataReceived(b"220 localhost\r\n")
+        transport.clear()
+        self.clientProtocol.dataReceived(b"500 not an esmtp server\r\n")
+        self.assertEqual(b"QUIT\r\n", transport.value())
+
+
+    def test_requireTLSAndHELOFallbackSucceedsIfOverTLS(self):
+        """
+        If TLS is provided at the transport level, we can honour the HELO
+        fallback if we're set to require TLS.
+        """
+        transport = StringTransport()
+        directlyProvides(transport, interfaces.ISSLTransport)
+        self.clientProtocol.requireAuthentication = False
+        self.clientProtocol.requireTransportSecurity = True
+        self.clientProtocol.heloFallback = True
+        self.clientProtocol.makeConnection(transport)
+
+        self.clientProtocol.dataReceived(b"220 localhost\r\n")
+        transport.clear()
+        self.clientProtocol.dataReceived(b"500 not an esmtp server\r\n")
+        self.assertEqual(b"HELO testuser\r\n", transport.value())
+
+
+
 class SSLTestCase(unittest.TestCase):
     """
     Tests for the TLS negotiation done by L{smtp.ESMTPClient}.
@@ -1740,7 +1818,7 @@ class AbortableStringTransport(StringTransport):
 
 
 
-class SendmailTestCase(unittest.TestCase):
+class SendmailTests(unittest.TestCase):
     """
     Tests for L{twisted.mail.smtp.sendmail}.
     """
@@ -1750,8 +1828,64 @@ class SendmailTestCase(unittest.TestCase):
         L{twisted.internet.reactor}.
         """
         args, varArgs, keywords, defaults = inspect.getargspec(smtp.sendmail)
-        index = len(args) - args.index("reactor") + 1
-        self.assertEqual(reactor, defaults[index])
+        self.assertEqual(reactor, defaults[2])
+
+
+    def test_honorsESMTPArguments(self):
+        """
+        L{twisted.mail.smtp.sendmail} creates the ESMTP factory with the ESMTP
+        arguments.
+        """
+        reactor = MemoryReactor()
+        smtp.sendmail("localhost", "source@address", "recipient@address",
+                      "message", reactor=reactor, username="foo",
+                      password="bar", requireTransportSecurity=True,
+                      requireAuthentication=True)
+        factory = reactor.tcpClients[0][2]
+        self.assertEqual(factory._requireTransportSecurity, True)
+        self.assertEqual(factory._requireAuthentication, True)
+        self.assertEqual(factory.username, "foo")
+        self.assertEqual(factory.password, "bar")
+
+
+    def test_messageFilePassthrough(self):
+        """
+        L{twisted.mail.smtp.sendmail} will pass through the message untouched
+        if it is a file-like object.
+        """
+        reactor = MemoryReactor()
+        messageFile = StringIO(b"File!")
+
+        smtp.sendmail("localhost", "source@address", "recipient@address",
+                      messageFile, reactor=reactor)
+        factory = reactor.tcpClients[0][2]
+        self.assertIs(factory.file, messageFile)
+
+
+    def test_messageStringMadeFile(self):
+        """
+        L{twisted.mail.smtp.sendmail} will turn non-file-like objects
+        (eg. strings) into file-like objects before sending.
+        """
+        reactor = MemoryReactor()
+        smtp.sendmail("localhost", "source@address", "recipient@address",
+                      "message", reactor=reactor)
+        factory = reactor.tcpClients[0][2]
+        messageFile = factory.file
+        messageFile.seek(0)
+        self.assertEqual(messageFile.read(), "message")
+
+
+    def test_senderDomainName(self):
+        """
+        L{twisted.mail.smtp.sendmail} passes through the sender domain name, if
+        provided.
+        """
+        reactor = MemoryReactor()
+        smtp.sendmail("localhost", "source@address", "recipient@address",
+                      "message", reactor=reactor, senderDomainName="foo")
+        factory = reactor.tcpClients[0][2]
+        self.assertEqual(factory.domain, "foo")
 
 
     def test_cancelBeforeConnectionMade(self):
